#include <stdio.h>
#include <stdlib.h> // abs()

/* b/a の分数を表す型 (a>0) */
typedef struct {
    int b; // 分子
    int a; // 分母
} frac_t;

// プロトタイプ宣言
int gcd(int x, int y);
void frac_reduce(frac_t *x);
void frac_add(frac_t *r, frac_t *x, frac_t *y);
void frac_sub(frac_t *r, frac_t *x, frac_t *y);
void frac_print(frac_t *x);

/* xとyの最大公約数を返す (x,y>=0 かつ x+y>0 を前提とする) */
int gcd(int x, int y) {
    while (y != 0) { // ユークリッドの互除法
        int t = x % y;  x = y;  y = t;
    }
    return x;
}

/* 約分 */
void frac_reduce(frac_t *x) {
    int d = gcd(abs(x->b), x->a);
    x->b /= d;
    x->a /= d;
}

/* 加算（*r = *x + *y） */
void frac_add(frac_t *r, frac_t *x, frac_t *y) {
    r->b = x->b * y->a + x->a * y->b;
    r->a = x->a * y->a;
    frac_reduce(r); // 約分を利用
}

/* 減算（*r = *x - *y） */
void frac_sub(frac_t *r, frac_t *x, frac_t *y) {
    frac_t tmp = { -y->b, y->a }; // 分子の符号を反転した分数を作る
    frac_add(r, x, &tmp);         // 加算を利用
}

/* 表示 */
void frac_print(frac_t *x) {
    printf("%d/%d\n", x->b, x->a);
}

int main(void) {
    frac_t x = { 7, 12 }; // 7/12
    frac_t y = { 1, 4 };  // 1/4
    frac_t r;             // 結果の代入先
    printf("add "); frac_add(&r, &x, &y); frac_print(&r); // 5/6
    printf("sub "); frac_sub(&r, &x, &y); frac_print(&r); // 1/3
    printf("0   "); frac_sub(&r, &x, &x); frac_print(&r); // 0/1
}
